/*
* Copyright © 2018 - 2019 Virtru Corporation
*
* SPDX - License - Identifier: MIT
*
*/
//
//  Virtru TDF3 SDK
//
//  Created by Pat Mancuso on 2019/05/06

#include <fstream>
#include <iostream>
#include <sstream>
#include <string>
#include <array>

using namespace std::string_literals;

// Only interface required.
#include <virtru_client.h>
#include <tdf3_constants.h>

using namespace virtru;

// User credentials
const std::string appId = "11111111-1111-1111-1111-111111111111";  // Put owner appId here
const std::string user = "your-user-name@your-domain.com";         // Put owner email here

// Sample file names and data
const char* samplePlaintextFilename = "samplePlaintext.txt";
const char* sampleCiphertextFilename = "sampleCiphertext.txt.html";
const char* sampleDecryptedFilename = "sampleDecrypted.txt";

const char* testTextData = "This is a simple test.";

// Function to create a data file to operate on
void createSampleData()
{
    // Housekeeping - clean start
    std::remove(samplePlaintextFilename);
    std::remove(sampleCiphertextFilename);
    std::remove(sampleDecryptedFilename);

    // Create a file to work on
    // This does not specify a path, and therefore operates on files in the current working directory
    std::ofstream samplePlaintextStream = std::ofstream(samplePlaintextFilename, std::ofstream::out);
    if (samplePlaintextStream.fail()) {
        throw std::runtime_error("Could not create input file");
    }
    else
    {
        // Write data to file and close it
        samplePlaintextStream << testTextData;
        samplePlaintextStream.close();
    }
}

// Encrypt/Decrypt using file interface.
void fileApiSample() {
    try {
        createSampleData();

        // Create an instance of the Virtru client.
        Client client {user, appId};

        // Use the client instance to encrypt a file
        auto policyId = client.encryptFile({samplePlaintextFilename, sampleCiphertextFilename});

        // Use the client instance to decrypt the file again
        client.decryptFile(sampleCiphertextFilename, sampleDecryptedFilename);

        std::cout << "Encrypt and decrypt file operations completed successfully" << std::endl;

        // Add a user
        Policy policy;
        policy.shareWithUsers({"someOtherUser@place.com"});

        // Set an expiration date in the future
        policy.addExpiration("2099-05-24");

        // Update the policy changes at the server
        client.updatePolicyForUUID(policy, policyId);

        std::cout << "Policy update operations completed successfully" << std::endl;
    }
    catch (const std::exception& exception) {
        std::cerr << "Exception " << exception.what() << std::endl;
    }
}

// Encrypt/Decrypt using string interface.
void stringApiSample() {
    try {
        auto plainText = "Hello World!!"s;

        // Create an instance of the Virtru client.
        Client client {user, appId};

        // Use the client instance to encrypt a string
        auto [policyId, tdfData]  = client.encryptString({plainText});

        // Use the client instance to decrypt the tdf data
        auto decryptText = client.decryptString(std::move(tdfData));

        if (plainText.compare(decryptText) == 0) {
            std::cout << "Encrypt and decrypt string api operations completed successfully" << std::endl;
        } else {
            std::cout << "Encrypt and decrypt string api operations failed" << std::endl;
        }

        // NOTE: Policy updates operations can be run on policyId.
    }
    catch (const std::exception& exception) {
        std::cerr << "Exception " << exception.what() << std::endl;
    }
}


// Encrypt/Decrypt using stream interface.
void streamApiSample() {
    try {
        auto plainText = "Virtru offers data protection solutions for the most commonly used "
                         "tools and applications across all industries"s;

        // Create an instance of the Virtru client.
        Client client {user, appId};

        std::stringstream plainTextStream(plainText);
        std::stringstream ioStream; // will be used as input and output stream

        // Use the client instance to encrypt a stream
        auto policyId = client.encryptStream({plainTextStream, ioStream});

        std::ostringstream outPlaintTextStream;

        // Decrypt the tdf out stream
        client.decryptStream(ioStream, outPlaintTextStream);

        if (plainText.compare(outPlaintTextStream.str()) == 0) {
            std::cout << "Encrypt and decrypt stream api operations completed successfully" << std::endl;
        } else {
            std::cout << "Encrypt and decrypt stream api operations failed" << std::endl;
        }

        // NOTE: Policy updates operations can be run on policyId.
    }
    catch (const std::exception& exception) {
        std::cerr << "Exception " << exception.what() << std::endl;
    }
}

// Encrypt/Decrypt using data callback interface.
void dataApiSample() {
    try {
        std::array<char, 5> buffer;
        auto plainText = "Virtru offers data protection solutions for the most commonly used "
                         "tools and applications across all industries"s;

        std::istringstream inputStream(plainText);
        auto encryptSourceCB = [&inputStream, &buffer](virtru::Status& status)->BufferSpan {
            if (inputStream.read(buffer.data(), buffer.size())) {
                status = Status::Success;
                return { (const std::uint8_t *) buffer.data(), buffer.size() };
            } else if (inputStream.eof()) {
                status = Status::Success;
                return { (const std::uint8_t *) buffer.data(), static_cast<std::size_t>(inputStream.gcount()) };
            } else {
                status = virtru::Status::Failure;
                return { nullptr, 0 };
            }
        };

        std::stringstream ioStream; // will be used as input and output stream
        auto encryptSinkCB = [&ioStream](BufferSpan bufferSpan) {
            if (!ioStream.write((const char*)(bufferSpan.data), bufferSpan.dataLength))
                return Status::Failure;
            else
                return virtru::Status::Success;
        };

        // Create an instance of the Virtru client.
        Client client {user, appId};

        EncryptDataParams encryptParams{encryptSourceCB, encryptSinkCB};

        // Encrypt the plaintext
        auto policyId = client.encryptData(encryptParams);

        auto decryptSourceCB = [&ioStream, &buffer](virtru::Status& status)->BufferSpan {
            if (ioStream.read(buffer.data(), buffer.size())) {
                status = Status::Success;
                return { (const std::uint8_t *) buffer.data(), buffer.size() };;
            }
            else if (ioStream.eof()) {
                status = Status::Success;
                return { (const std::uint8_t *) buffer.data(), static_cast<std::size_t>(ioStream.gcount()) };
            }
            else {
                status = virtru::Status::Failure;
                return { nullptr, 0 };
            }
        };

        std::string decryptedText;
        auto decryptSinkCB = [&decryptedText](BufferSpan bufferSpan) {
            decryptedText.append((const char*)bufferSpan.data, bufferSpan.dataLength);
            return virtru::Status::Success;
        };

        client.decryptData(decryptSourceCB, decryptSinkCB);

        if (plainText.compare(decryptedText) == 0) {
            std::cout << "Encrypt and decrypt data api operations completed successfully" << std::endl;
        } else {
            std::cout << "Encrypt and decrypt data api operations failed" << std::endl;
        }

        // NOTE: Policy updates operations can be run on policyId.
    }
    catch (const std::exception& exception) {
        std::cerr << "Exception " << exception.what() << std::endl;
    }
}


// Simple application to encrypt and decrypt files using the Virtru SDK for C++
int main()
{
    // File api.
    fileApiSample();

    // String api
    stringApiSample();

    // Stream api
    streamApiSample();

    // Data api
    dataApiSample();

    return 0;
}
